/*
 * serial.c
 *
 *  Created on: 07.06.2019
 *      Author: harry
 */

#include <string.h>
#include "usart.h"

#include "fifo.h"
#include "serial.h"

#define SERIAL_ECHO

static serial_rx_buf_t EditBuffer;
static uint8_t tx_buffer[TX_BUFFER_SIZE];
static serial_rx_buf_t *TargetBuffer;
static volatile uint8_t uart_idle;
static volatile uint8_t line_complete;

static fifo_t tx_fifo;
static uint8_t buffer_p;
static uint8_t rxbuf;

static UART_HandleTypeDef *hcomport;

/********************************************************************
 * Serial API-Fuctions
 ********************************************************************/

void serial_init(UART_HandleTypeDef *huart, serial_rx_buf_t *LineBuffer)
{
	hcomport = huart;
	fifo_init(&tx_fifo, (uint8_t *) &tx_buffer, TX_BUFFER_SIZE);
	line_complete = FALSE;
	uart_idle = TRUE;
	EditBuffer[0] = 0;
	buffer_p = 0;
	TargetBuffer = LineBuffer;
	HAL_UART_Receive_IT(hcomport, &rxbuf, 1);
}

uint8_t serial_isLineAvailable(void)
{
	if (line_complete)
	{
		line_complete = FALSE;
		return TRUE;
	}
	else
		return FALSE;
}

void serial_putc(char c)
{
	static uint8_t ci;

	while (!fifo_put(&tx_fifo, c))
		;
	if (uart_idle == TRUE)
	{
		uart_idle = FALSE;
		fifo_get(&tx_fifo, &ci);
		HAL_UART_Transmit_IT(hcomport, (uint8_t *) &ci, 1);
	}
}

void serial_puts(char *s)
{
	for (uint8_t i = 0; i < strlen(s); i++)
		serial_putc(s[i]);
}

/********************************************************************
 * Serial	Callback
 * called, when a line is completed by typing <CR> and the line is
 * copied to the TargetBuffer
 ********************************************************************/

__weak void uart_LineReceivedCallback(serial_rx_buf_t *line)
{
	UNUSED (line);
}

/********************************************************************
 * Serial Iterrupt-Handler
 ********************************************************************/

void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart)
{
	if (huart == hcomport)
	{
		if (rxbuf >= ' ')
		{
			if (buffer_p < (SERIAL_RX_BUFFER_SIZE-1))
			{
				EditBuffer[buffer_p++] = rxbuf;
				EditBuffer[buffer_p] = '\0';
#ifdef SERIAL_ECHO
				serial_putc(rxbuf);
#endif
			}
		}
		else
		{
			switch (rxbuf)
			{
			case '\r':
				// got CR
				serial_putc(rxbuf);
				EditBuffer[buffer_p] = 0;
				buffer_p = 0;
#ifdef SERIAL_ECHO
				serial_puts("\r\n");
#endif
				strcpy((char*)TargetBuffer, (char*)EditBuffer);
				line_complete = TRUE;
				uart_LineReceivedCallback(TargetBuffer);

				break;

			case '\b':			// backspace
#ifdef SERIAL_ECHO
				if (buffer_p)
				{
					serial_puts("\b \b");
					buffer_p--;
				}
				else
					serial_putc(0x07);	// beep
#endif
				break;

			case '\e':			// escape
				buffer_p = 0;
				// send singele escape
				break;
			default:
				break;
			}
		}
		HAL_UART_Receive_IT(hcomport, &rxbuf, 1);
	}
}

void HAL_UART_TxCpltCallback(UART_HandleTypeDef *huart)
{
	static uint8_t c;

	if (huart == hcomport)
	{
		if (fifo_get(&tx_fifo, &c))
		{
			HAL_UART_Transmit_IT(hcomport, (uint8_t *) &c, 1);
		}
		else
			uart_idle = TRUE;
	}

}


